/*********************************************************************/
/*                                                                   */
/*   FAXNUM.C                                                        */
/*                                                                   */
/*   OV/Fax Server Fax Number Processor                              */
/*                                                                   */
/*   This program is supplied as a sample user exit program, for     */
/*   the purpose of processing fax numbers prior to dialling.  It    */
/*   is intended for use in North America, where the standard        */
/*   number processing routines of the server may not be adequate.   */
/*                                                                   */
/*   The supplied EXE program was compiled using the Microsoft VC++  */
/*   compiler, but the source should be compatible with most         */
/*   C compilers.                                                    */
/*                                                                   */
/*   Copyright Icom Solutions Ltd. 1996                              */
/*                                                                   */
/*   'This code constitutes a sample program and is distributed on   */
/*   an "as is" basis without any warrantry either express or        */
/*   implied.                                                        */
/*   While the code may have been formally tested in a specific      */
/*   environment there is no guarantee that the same or similar      */
/*   results will be obtained elsewhere.  Customers attempting to    */
/*   use or adapt the code or the techniques described or used       */
/*   herein to their own environment do so at their own risk.'       */
/*                                                                   */
/*   V1.00     04/07/95  (PCR040) Original version written.          */
/*   V1.00     17/12/95  Disclaimer added.                           */
/*   V1.00     10/01/96  Amended copyright.                          */
/*   V2.40     29/02/96  Amended version number.                     */
/*                                                                   */
/*********************************************************************/

#include <fcntl.h> 
#include <stdio.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <share.h>
#include <io.h>
#include <stdlib.h>
#include <dos.h>
#include <process.h>
#include <ctype.h>
#include <string.h>
#include <direct.h>
#include <memory.h>

char copyright [] = "Copyright Icom Solutions Ltd 1996.";
char version [] = "V2.4";

/*********************************************************************/
/* function declarations */

int  main ( int, char *[] );
int  parse_parms ( char [] );
int  check_country ( void );
void search_table ( void );
int  write_file ( void );

/*********************************************************************/

/* parameters */

char line [16];          /* line prefix */
char natl [16];          /* national prefix */
char area [16];          /* local area code */
char intl [16];          /* international dialling code */
char ctry [16];          /* country code */

int  lenl = 0;           /* length of line prefix */
int  lenn = 0;           /* length of national prefix */
int  lena = 0;           /* length of local area code */
int  leni = 0;           /* length of intl dialling code */
int  lenc = 0;           /* length of country code */

char faxnum [40];        /* fax number */
char newfaxnum [40];     /* new fax number to write to file */

/*********************************************************************/
/*                                                                   */
/*   MAIN                                                            */
/*                                                                   */
/*   This is the main module of the program.                         */
/*                                                                   */
/*   main ( argc, argv )                                             */
/*                                                                   */
/*   int argc;      number of arguments to program                   */
/*   char *argv[];  array of pointers to arguments                   */
/*                                                                   */
/*   Arguments:     0 = program name                                 */
/*                  1 = line/nat'l/area/int'l/country/fax-number     */
/*                                                                   */
/*   Returns:                                                        */
/*                                                                   */
/*        0    fax number processed and result file created          */
/*        1    no result file created                                */
/*                                                                   */
/*********************************************************************/

int main ( argc, argv )
int argc;
char *argv[];
     {
     /* process the parameters to extract the dialling prefixes and fax number */

     if ( argc < 2 )
          return 1;           /* no parameters supplied */

     if ( parse_parms ( argv[1] ) != 0 )
          return 1;           /* invalid parameters supplied */

     /* check that we can process the number */

     if ( check_country () != 0 )
          return 1;           /* another country so cannot process here */

     /* generate a new number and write to the file */

     search_table ();

     if ( write_file () != 0 )
          return 1;           /* error writing replacement number */

     return ( 0 );
     }

/*********************************************************************/
/*                                                                   */
/*   PARSE_PARMS                                                     */
/*                                                                   */
/*   This routine parses the command line parms to extract the       */
/*   number prefixes and fax number.                                 */
/*                                                                   */
/*   int parse_parms ( argv );                                       */
/*                                                                   */
/*   char argv[];   command line parms (separated by slashes)        */
/*                                                                   */
/*   Returns:                                                        */
/*                                                                   */
/*        0    successful                                            */
/*        1    error in the parms                                    */
/*                                                                   */
/*********************************************************************/

int parse_parms ( argv )
char argv[];
     {
     int s, d, len;

     len = strlen ( argv );

     /* extract the line prefix */
     for ( s = 0, lenl = 0; (lenl < 16) && (argv[s] != '/'); s++ )
          line [lenl++] = argv [s];
     line [lenl] = '\0';
     if ( s > len )
          return 1;

     /* extract the national prefix */
     for ( s++, lenn = 0; (lenn < 16) && (argv[s] != '/'); s++ )
          natl [lenn++] = argv [s];
     natl [lenn] = '\0';
     if ( s > len )
          return 1;

     /* extract the local area code */
     for ( s++, lena = 0; (lena < 16) && (argv[s] != '/'); s++ )
          area [lena++] = argv [s];
     area [lena] = '\0';
     if ( s > len )
          return 1;

     /* extract the international prefix */
     for ( s++, leni = 0; (leni < 16) && (argv[s] != '/'); s++ )
          intl [leni++] = argv [s];
     intl [leni] = '\0';
     if ( s > len )
          return 1;

     /* extract the country code */
     for ( s++, lenc = 0; (lenc < 16) && (argv[s] != '/'); s++ )
          ctry [lenc++] = argv [s];
     ctry [lenc] = '\0';
     if ( s > len )
          return 1;

     /* extract the fax number, removing all but digits and country specifier */
     for ( s++, d = 0; s < len; s++ )
          if ( isdigit ( argv [s] ) || argv [s] == '.' )
               faxnum [d++] = argv [s];
     faxnum [d] = '\0';
     if ( d < 1 )
          return 1;

     return 0;
     }

/*********************************************************************/
/*                                                                   */
/*   CHECK_COUNTRY                                                   */
/*                                                                   */
/*   This routine parses the fax number to check if it is for        */
/*   another country.  If so, the server's own default number        */
/*   routines will be used.  If it this country, any international   */
/*   or country code prefix will be removed.                         */
/*                                                                   */
/*   int check_country ( void );                                     */
/*                                                                   */
/*   Returns:                                                        */
/*                                                                   */
/*        0    this country, no international/country prefixes       */
/*        1    another country specified                             */
/*                                                                   */
/*********************************************************************/

int check_country ()
     {
     int s, d = 0;

     if ( strncmp ( faxnum, intl, leni ) == 0 )             /* international number */
          {
          if ( strncmp ( faxnum+leni, ctry, lenc ) != 0 )
               return 1;                                    /* not this country */
          else
               {
               s = leni + lenc;
               while ( faxnum [d++] = faxnum [s++] );       /* remove intl/ctry */
               }
          }

     else if ( strchr ( faxnum, '.' ) != NULL )             /* country specifier */
          {
          if ( ( strncmp ( faxnum, ctry, lenc ) != 0 )
            || ( faxnum [lenc] != '.' ) )
               return 1;                                    /* not this country */
          else
               {
               s = lenc + 1;
               while ( faxnum [d++] = faxnum [s++] );       /* remove 'ctry.' */
               }
          }

     /*
     * the fax is for this country and has now had any country prefix removed.
     * Further processing may be required here to remove any prefixes or suffixes
     * unique to the implementation, for example, account codes appended to the
     * fax number
     */

     return 0;
     }

/*********************************************************************/
/*                                                                   */
/*   SEARCH_TABLE                                                    */
/*                                                                   */
/*   This routine scans through the mapping table, attempting to     */
/*   match the fax number with a map entry.  If found, the number    */
/*   can be treated as local, but there may be digits to replace.    */
/*   The initial fax number has any existing long distance code      */
/*   removed, or the local area code added if not specified.         */
/*                                                                   */
/*   Note that this part of the program is very specific to the      */
/*   North American telephone systems.  The mapping table should     */
/*   the first digit in the first byte of each line.  Replacement    */
/*   values follow the search number, with only an equals sign to    */
/*   separate them.                                                  */
/*                                                                   */
/*   void check_country ( void );                                    */
/*                                                                   */
/*   Returns:                                                        */
/*                                                                   */
/*        nothing                                                    */
/*                                                                   */
/*********************************************************************/

void search_table ()
     {
     int s = 0, d = 0, n = 0;
     char search_num [40], line [128];
     FILE *FPtbl;

     /*
     * set up the number to search the table with
     */

     if ( strlen ( faxnum ) < 8 )                           /* no area code specified */
          sprintf ( search_num, "%s%s", area, faxnum );     /* so prefix with area code */

     else
          {
          if ( faxnum [0] == '1' )
               s = 1;                                       /* remove long distance code */
          while ( search_num [d++] = faxnum [s++] );        /* copy to search_num */
          }

     /*
     * read through the table, looking for a match
     */

     if ( ( FPtbl = fopen ( "FAXNUM.MAP", "rt" ) ) != NULL )
          {
          while ( fgets ( line, 127, FPtbl ) != NULL )
               {
               if ( line [0] == ';' )
                    continue;                                    /* comment line */

               s = 0;
               while ( isdigit ( line [s] ) )                    /* get the number to check */
                    s++;

               if ( s == 0 )
                    continue;                                    /* not a valid line */

               if ( strncmp ( search_num, line, s ) == 0 )       /* match found */
                    {
                    if ( line [s] == '=' )                       /* replacement value */
                         {
                         d = s + 1;
                         while ( isdigit ( line [d] ) )
                              newfaxnum [n++] = line [d++];
                         strcpy ( newfaxnum + n, search_num + s );
                         }

                    else
                         strcpy ( newfaxnum, search_num );       /* local call */

                    return;
                    }

               /* no match so get the next line */
               }    /* end while */
          fclose ( FPtbl );
          }

     /*
     * no match found, or table missing, so treat as a long distance call
     */

     sprintf ( newfaxnum, "1%s", search_num );

     return;
     }

/*********************************************************************/
/*                                                                   */
/*   WRITE_FILE                                                      */
/*                                                                   */
/*   This routine writes the results file, prefixing the generated   */
/*   number with a line prefix if specified by the server.           */
/*                                                                   */
/*   int write_file ( void );                                        */
/*                                                                   */
/*   Returns:                                                        */
/*                                                                   */
/*        0    file written                                          */
/*        1    error creating file                                   */
/*                                                                   */
/*********************************************************************/

int write_file ()
     {
     FILE *FPres;

     if ( ( FPres = fopen ( "FAXNUM.RES", "wb" ) ) == NULL )
          return 1;

     fprintf ( FPres, "%s%s", line, newfaxnum );
     fclose ( FPres );

     return 0;
     }

/*********************************************************************/
/*   END OF FAXNUM.C                                                 */
/*********************************************************************/
